; This program is free software. It comes without any warranty, to
; the extent permitted by applicable law. You can redistribute it
; and/or modify it under the terms of the Do What The Fuck You Want
; To Public License, Version 2, as published by Sam Hocevar. See
; http://sam.zoy.org/wtfpl/COPYING for more details.


;------ IncPtr -----------------------------------------------------------------
IncPtr:
; Increments CHL, wrapping as needed.  If HL points outside of the swappable
; memory bank, this does not attempt any wrapping.
; Input:
;  - CHL
; Output:
;  - CHL+1
; Destroys:
;  - AF
	inc	l
	ret	nz
	inc	h
	ld	a, h
	cp	swapTop >> 8
	ret	nz
	sub	40h
	ld	h, a
	inc	c
	ret


;------ AddPtr -----------------------------------------------------------------
AddPtr:
; Adds DE to CHL, wrapping as needed.  THIS WILL SCREW UP if HL points outside
; of the swap bank.
; Inputs:
;  - CHL: Pointer
;  - DE: Amount to add to pointer
; Output:
;  - CHL += DE
; Destroys:
;  - AF
#if swapLoc != 4000h
	.error	"Oops, fix the AddPtr function.  It assumes swapping happens at 4000h."
#endif
#IFDEF never
	; Lower byte
	ld	a, l
	add	a, e
	ld	l, a
	; Upper byte
	ld	a, h
	sub	40h
	add	a, d
	ld	h, a
	; Page
	rlca
	rlca
	add	a, c
	ld	c, a
	ld	a, h
	add	a, 40h
	ld	h, a
	ret
#endif
;#IFDEF never
	add	hl, de
	ld	a, h
	cp	40h
	jr	nc, +_
	sub	40h
	inc	c
_:	cp	0C0h
	jr	c, +_
	sub	40h
	inc	c
_:	cp	80h
	jr	c, +_
	sub	40h
	inc	c
_:	ld	h, a
	ret
;#endif


;------ GetByte ----------------------------------------------------------------
GetByte:
; Gets a byte from the given location.
; Input:
;  - CHL: Address
; Output:
;  - A: Byte returned
; Destroys:
;  - B
	; Save page
	in	a, (swapPortHigh)
	rra
	in	a, (swapPort)
	push	af
	; Write page
	ld	a, c
	rlca
	out	(swapPortHigh), a
	srl	a
	out	(swapPort), a
	; Fetch data
	ld	b, (hl)
	; Restore page
	pop	af
	out	(swapPort), a
;	ld	a, 0
;	adc	a, a
	sbc	a, a	; If this causes problems (which it will do somewhat consistently), use the above
	out	(swapPortHigh), a
	; Return data
	ld	a, b
	ret


;------ GetByteIncPtr ----------------------------------------------------------
GetByteIncPtr:
; Calls GetByte and IncPtr.
	call	GetByte
	push	af
	call	IncPtr
	pop	af
	ret


;------ GetBytes ---------------------------------------------------------------
GetBytes:
; Copies bytes from flash
; Inputs:
;  - CHL: Source
;  - DE: Destination
;  - B: Number of bytes to copy
; Output:
;  - Data copied
; Destroys:
;  - B
	push	bc
	call	GetByte
	ld	(de), a
	inc	de
;	call	DispByte
	call	IncPtr
	pop	bc
	djnz	GetBytes
	ret